local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

a.spells["Auto Shot"] = {
	ID = 75--[[Auto Shot]],
	EnemyTargetNeeded = true,
	NotIfActive = true,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "Green", "Pink")
		return x.NoCC and not s.Shooting()
	end,
	NoGCD = true,
}

a.spells["Pet Attack"] = {
	Override = function(z)
		if x.NoCC and x.PetAlive and x.Enemy and not x.PetActiveEnemy then
			s.FlashPet(PET_ACTION_ATTACK, "Green")
		elseif x.NoCC and x.PetAlive and x.Enemy and x.PetActiveEnemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK, "Pink")
		end
	end,
}
a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and not a.PetAssistActive() then
			s.FlashPet(PET_MODE_ASSIST, "Pink")
		end
	end,
}

a.spells["Hunter's Mark"] = {
	ID = 1130--[[Hunter's Mark]],
	Debuff = 1130--[[Hunter's Mark]],
	EarlyRefresh = 5,
	Check = function(z)
		return x.ShouldTemporaryBuff and not s.HasGlyph(132106--[[Glyph of Marked for Death]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aspect of the Hawk"] = {
	Type = "form",
	ID = 13165--[[Aspect of the Hawk]],
	Buff = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
	BuffUnit = "player",
	NotWhileMoving = true,
	NoGCD = true,
	FlashColor = "Yellow",
	Check = function(z)
		return x.ShouldTemporaryBuff and (not s.Moving("player") or s.InCombat())
	end,
	FlashID = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
}


a.spells["Aspect of the Cheetah while moving"] = {
	Type = "form",
	ID = 5118--[[Aspect of the Cheetah]],
	MyBuff = 5118--[[Aspect of the Cheetah]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldPermanentBuff and s.Moving("player") and not s.InCombat()
	end,
}

a.spells["Explosive Trap (AoE)"] = {
	ID = 13813--[[Explosive Trap]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	Run = function(z)
		z.Continue = s.If(a.AoE(), false, true)
		z.FlashColor = s.If(a.AoE(), "White", "Purple")
	end,
	FlashID = {
		13813--[[Explosive Trap]],
		82939--[[Explosive Trap]],
	},
	NoRangeCheck = true,
}

a.spells["Focus Fire at 5 stacks"] = {
	ID = 82692--[[Focus Fire]],
	Buff = 82692--[[Focus Fire]],
	BuffUnit = "player",
	RequireBuff = 19623--[[Frenzy]],
	RequireStack = 5,
	RequireBuffUnit = "pet",
	Check = function(z)
		return x.ActiveEnemy and x.PetAlive 
	end,
}

a.spells["Serpent Sting if not ticking"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]]) + (s.LastSpellTravelTime(77767--[[Cobra Shot]]) or 1) + x.Lag
		return x.NoCC and not s.Casting(77767--[[Cobra Shot]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Serpent Sting if not ticking and target HP <= 80%"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	EarlyRefresh = 1,
	CheckLast = function(z)
		return x.NoCC and (s.HealthPercent("target") <= 80 or s.Dummy("target"))
	end,
	EnemyTargetNeeded = true,
}

a.spells["Serpent Sting if not ticking and TTD  >= 10"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Continue = false,
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 10
	end,
	CheckLast = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]]) + (s.LastSpellTravelTime(77767--[[Cobra Shot]]) or 0)
		return not s.Casting(77767--[[Cobra Shot]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Chimera Shot"] = {
	ID = 53209--[[Chimera Shot]],
	CheckLast = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Bestial Wrath if focus > 60"] = {
	ID = 19574--[[Bestial Wrath]],
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and x.Focus > 60 
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Multi-Shot (AoE)"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if a.AoE() then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Purple"
			z.Continue = true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot (AoE)"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.AoE()
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot (AoE)"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC and a.AoE()
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Kill Shot"] = {
	ID = 53351--[[Kill Shot]],
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["A Murder of Crows"] = {
	ID = 131894--[[A Murder of Crows]],
	Check = function(z)
		return x.NoCC and s.HasTalent(131894--[[A Murder of Crows]]) and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Fervor with focus <= 50"] = {
	ID = 82726--[[Fervor]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(82726--[[Fervor]]) and x.Focus <= 50
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Fervor with focus <= 65"] = {
	ID = 82726--[[Fervor]],
	Check = function(z)
		return x.ActiveEnemy and s.HasTalent(82726--[[Fervor]]) and x.Focus <= 65
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Powershot"] = {
	ID = 109259--[[Powershot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and s.HasTalent(109259--[[Powershot]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Barrage"] = {
	ID = 120360--[[Barrage]],
	NotWhileMoving = false,
	NoStopChannel = true,
	Check = function(z)
		return x.NoCC and s.HasTalent(120360--[[Barrage]])
	end,
}

a.spells["Blink Strike"] = {
	ID = 130392--[[Blink Strike]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.PetAlive and s.HasTalent(130392--[[Blink Strike]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Lynx Rush"] = {
	ID = 120697--[[Lynx Rush]],
	NotWhileMoving = false,
	Check = function(z)
		return x.PetActiveEnemy and x.PetAlive and s.HasTalent(120697--[[Lynx Rush]])
	end,
	NoRangeCheck = true,
	EnemyTargetNeeded = false,
}

a.spells["Stampede w/ Rapid Fire or Bloodlust"] = {
	ID = 121818--[[Stampede]],
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.NoCC and (x.Bloodlust or x.RapidFire)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Glaive Toss"] = {
	ID = 117050--[[Glaive Toss]],
	Check = function(z)
		return x.NoCC and s.HasTalent(117050--[[Glaive Toss]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Dire Beast"] = {
	ID = 120679--[[Dire Beast]],
	Check = function(z)
		return x.NoCC and s.HasTalent(120679--[[Dire Beast]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Dire Beast when focus <= 90"] = {
	ID = 120679--[[Dire Beast]],
	Check = function(z)
		return x.NoCC and x.Focus <= 90 
	end,
	CheckLast = function(z)
		return s.HasTalent(120679--[[Dire Beast]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Kill Command"] = {
	ID = 34026--[[Kill Command]],
	Check = function(z)
		return x.PetNoCC and x.PetAlive
	end,
	PetFrameNeeded = true,
	EnemyTargetNeeded = true,
	NoRangeCheck = true,
}

a.spells["Arcane Shot with Thrill of the Hunt"] = {
	ID = 3044--[[Arcane Shot]],
	RequireBuff = 34720--[[Thrill of the Hunt]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "Yellow",
	EnemyTargetNeeded = true,
}

a.spells["Multi-Shot with Thrill of the Hunt and SS < 2"] = {
	ID = 2643--[[Multi-Shot]],
	RequireBuff = 34720--[[Thrill of the Hunt]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and s.MyDebuffDuration(1978--[[Serpent Sting]]) < 2
	end,
	FlashColor = "Yellow",
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot as focus dump when not hasted"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not a.PreSteadyFocus() and (not a.Hasted() or s.Moving("player")) and a.CanStillCast(53209--[[Chimera Shot]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot if focus >= 69 or The Beast Within up"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return a.CanStillCast(34026--[[Kill Command]]) or s.Buff(34692--[[The Beast Within]], "player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot if focus >= 67"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		if x.TimeToDie >= 8 then
			return a.CanStillCast({53301--[[Explosive Shot]], 3674--[[Black Arrow]]})
		elseif x.TimeToDie < 8 and x.TimeToDie > 2 then
			return a.CanStillCast(53301--[[Explosive Shot]])
		else
			return true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot with Pre Steady Focus and Steady Focus <= 5"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.PreSteadyFocus() and s.MyBuffDuration(53224--[[Steady Focus]], "player") < 5
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot with Pre Steady Focus and Steady Focus < 6"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.PreSteadyFocus() and s.MyBuffDuration(53224--[[Steady Focus]], "player") < 6
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot if Steady Focus < Cast Time"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and s.MyBuffDuration(53224--[[Steady Focus]], "player") < s.CastTime(53224--[[Steady Focus]]) and not s.SpellDelay(56641--[[Steady Shot]], "target")
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot BM"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	CheckLast = function(z)
		if x.Focus < s.SpellCost(34026--[[Kill Command]], SPELL_POWER_FOCUS) then
			return true
		end
		return (s.SpellCooldown(34026--[[Kill Command]]) > x.DoubleLag)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot SV"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	CheckLast = function(z)
		if x.Focus < s.SpellCost(53301--[[Explosive Shot]], SPELL_POWER_FOCUS) then
			return true
		end
		return (s.SpellCooldown(53301--[[Explosive Shot]]) > x.DoubleLag)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot if SS < 6"] = {
	ID = 77767--[[Cobra Shot]],
	RequireMyDebuff = 1978--[[Serpent Sting]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	CheckLast = function(z)
		return s.MyDebuffDuration(1978--[[Serpent Sting]]) < 6
	end,
	EnemyTargetNeeded = true,
}

a.spells["Rapid Fire"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	Continue = true,
	NoGCD = true,
	FlashColor = "Pink",
}

a.spells["Rapid Fire without Haste buffs or TTD <= 30"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and (not a.Hasted() or (x.TimeToDie <= 30))
	end,
	Continue = true,
	NoGCD = true,
	FlashColor = "Pink",
}

a.spells["Readiness after Rapid Fire"] = {
	ID = 23989--[[Readiness]],
	Check = function(z)
		return x.NoCC and s.HasTalent(109215--[[Posthaste]]) and s.SpellCooldown(3045--[[Rapid Fire]]) > x.TimeToDie
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Aimed Shot with Master Marksman"] = {
	ID = 19434--[[Aimed Shot]],
	RequireMyBuff = 82926--[[Fire!]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	FlashColor = "Yellow",
	EnemyTargetNeeded = true,
}

a.spells["Aimed Shot"] = {
	ID = 19434--[[Aimed Shot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aimed Shot when hasted"] = {
	ID = 19434--[[Aimed Shot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not a.PreSteadyFocus() and a.Hasted()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Call Pet"] = {
	ID = 9--[[Call Pet]],
	Override = function(z)
		if not x.PetAlive and x.ShouldPermanentBuff then
			s.Flash({
				9--[[Call Pet]],
				883--[[Call Pet 1]],
				83242--[[Call Pet 2]],
				83243--[[Call Pet 3]],
				83244--[[Call Pet 4]],
				83245--[[Call Pet 5]],
				982--[[Revive Pet]]})
		end
	end,
}

a.spells["Mend Pet"] = {
	ID = 136--[[Mend Pet]],
	Buff = 136--[[Mend Pet]],
	BuffUnit = "pet",
	EarlyRefresh = 1,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.PetAlive and (s.HealthPercent("pet") < 80 or (s.HasGlyph(19573--[[Glyph of Mend Pet]]) and s.Debuff(nil, "pet", nil, nil, 1)))
	end,
}

a.spells["Explosive Shot with Lock and Load"] = {
	ID = 53301--[[Explosive Shot]],
	RequireBuff = 56453--[[Lock and Load]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "Yellow",
	EnemyTargetNeeded = true,
}

a.spells["Explosive Shot"] = {
	ID = 53301--[[Explosive Shot]],
	Check = function(z)
		return x.NoCC
	end,
	FlashColor = "White",
	EnemyTargetNeeded = true,
}

a.spells["Black Arrow if not ticking and TTD  >= 8"] = {
	ID = 3674--[[Black Arrow]],
	MyDebuffID = 3674--[[Black Arrow]],
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 8
	end,
	EnemyTargetNeeded = true,
}

a.spells["Silencing Shot"] = {
	ID = 34490--[[Silencing Shot]],
	Debuff = 34490--[[Silencing Shot]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoGCD = true,
	FlashColor = "Aqua",
}

a.spells["Potion with Bloodlust or TTD <= 60"] = {
	Type = "item",
	ID = 76089--[[Virmen's Bite]],
	Buff = {105697--[[Virmen's Bite]], 79633--[[Potion of the Tol'vir]]},
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and s.Boss("target")
	end,
	CheckLast = function(z)
		return x.Bloodlust or (x.TimeToDie <= 60)
	end,
	FlashID = {76089--[[Virmen's Bite]], 58145--[[Potion of the Tol'vir]]},
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Concussive Shot"] = {
	ID = 5116--[[Concussive Shot]],
	CheckLast = function(z)
		return not s.Boss("target") and s.Moving("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	EnemyTargetNeeded = true,
}

a.spells["Disengage"] = {
	ID = 781--[[Disengage]],
	CheckLast = function(z)
		return (x.ActiveEnemy and s.MeleeDistance("target")) or (s.MovementImpaired("player") and s.HasTalent(109215--[[Posthaste]]))
	end,
	FlashColor = "Yellow",
	NoGCD = true,
	NoRangeCheck = true,
	EnemyTargetNeeded = true,
}

a.spells["Feign Death"] = {
	ID = 5384--[[Feign Death]],
	NotWhileMoving = true,
	CheckLast = function(z)
		return (s.HealthPercent("player") < 50 and s.EnemyTargetingYou("target")) or x.ThreatPercent > 80
	end,
	FlashColor = "Red",
	NoGCD = true,
}

a.spells["Master's Call"] = {
	ID = 53271--[[Master's Call]],
	CheckLast = function(z)
		return x.PetAlive and s.MovementImpaired("player") or s.MovementImpaired("pet")
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Widow Venom"] = {
	ID = 82654--[[Widow Venom]],
	Debuff = {
		82654--[[Widow Venom]],
		8680--[[Wound Poison]],
		115804--[[Mortal Wounds]],
		54680--[[Monstrous Bite]],
	},
	CheckLast = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Pink",
}

a.spells["Exhilaration"] = {
	ID = 109304--[[Exhilaration]],
	CheckLast = function(z)
		return a.HasTalent(109304--[[Exhilaration]]) and (s.HealthPercent("player") < 70 or (x.PetAlive and s.HealthPercent("pet") < 50))
	end,
	NoGCD = true,
	FlashColor = "Red",
}